<?php

namespace Drupal\language_combination\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\DataReferenceDefinition;
use Drupal\language\Entity\ConfigurableLanguage;

/**
 * Plugin implementation of the 'language_combination' field type.
 *
 * @FieldType(
 *   id = "language_combination",
 *   label = @Translation("Language Combination"),
 *   description = @Translation("Allows the definition of language combinations (e.g. 'From english to german')."),
 *   default_widget = "language_combination_default",
 *   default_formatter = "language_combination_default",
 *   constraints = {"LTLanguageCombination" = {}},
 *   multiple = true,
 * )
 */
class LanguageCombination extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field) {
    $property_definitions['source_languages'] = DataDefinition::create('string')
      ->setLabel(t('From language'));
    $property_definitions['target_languages'] = DataDefinition::create('string')
      ->setLabel(t('To language'));

    $properties['language_from'] = DataReferenceDefinition::create('language')
      ->setLabel(t('Language from object'))
      ->setDescription(t('The referenced language'))
      // The language object is retrieved via the language code.
      ->setComputed(TRUE)
      ->setReadOnly(FALSE);

    $properties['language_to'] = DataReferenceDefinition::create('language')
      ->setLabel(t('Language to object'))
      ->setDescription(t('The referenced language'))
      // The language object is retrieved via the language code.
      ->setComputed(TRUE)
      ->setReadOnly(FALSE);


    return $property_definitions;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field) {
    return [
      'columns' => [
        'source_languages' => [
          'description' => 'The langcode of the language from which the user is able to translate.',
          'type'        => 'varchar',
          'length'      => 10,
        ],
        'target_languages' => [
          'description' => 'The langcode of the language to which the user is able to translate.',
          'type'        => 'varchar',
          'length'      => 10,
        ],
      ],
      'indexes' => [
        'language' => ['source_languages', 'target_languages'],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function setValue($values, $notify = TRUE) {
    if (isset($values['source_languages']) && !isset($values['language_from'])) {
      $values['language_from'] = $values['source_languages'];
    }
    if (isset($values['target_languages']) && !isset($values['language_to'])) {
      $values['language_to'] = $values['target_languages'];
    }
    parent::setValue($values, $notify);
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    return empty($this->language_from)
      || empty($this->language_to)
      || $this->language_from == '_none'
      || $this->language_to == '_none';
  }

  /**
   * {@inheritdoc}
   */
  public function preSave() {
    parent::preSave();

    // In case the skill languages is not know to the system, install them.
    $languages = \Drupal::languageManager()->getLanguages();
    if (!isset($languages[$this->language_from])) {
      $language = ConfigurableLanguage::createFromLangcode($this->language_from);
      $language->save();
    }
    if (!isset($languages[$this->language_to])) {
      $language = ConfigurableLanguage::createFromLangcode($this->language_to);
      $language->save();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onChange($property_name, $notify = TRUE) {
    // Make sure that the value and the language property stay in sync.
    if ($property_name == 'source_languages') {
      $this->writePropertyValue('language_from', $this->source_languages);
    }
    if ($property_name == 'target_languages') {
      $this->writePropertyValue('language_to', $this->target_languages);
    }
    if ($property_name == 'language_from') {
      $this->writePropertyValue('source_languages', $this->get('language_from')->getTargetIdentifier());
    }
    if ($property_name == 'language_to') {
      $this->writePropertyValue('target_languages', $this->get('language_to')->getTargetIdentifier());
    }
    parent::onChange($property_name, $notify);
  }

}
